/*
 *
 * Copyright (c) 2017 Texas Instruments Incorporated
 *
 * All rights reserved not granted herein.
 *
 * Limited License.
 *
 * Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive
 * license under copyrights and patents it now or hereafter owns or controls to make,
 * have made, use, import, offer to sell and sell ("Utilize") this software subject to the
 * terms herein.  With respect to the foregoing patent license, such license is granted
 * solely to the extent that any such patent is necessary to Utilize the software alone.
 * The patent license shall not apply to any combinations which include this software,
 * other than combinations with devices manufactured by or for TI ("TI Devices").
 * No hardware patent is licensed hereunder.
 *
 * Redistributions must preserve existing copyright notices and reproduce this license
 * (including the above copyright notice and the disclaimer and (if applicable) source
 * code license limitations below) in the documentation and/or other materials provided
 * with the distribution
 *
 * Redistribution and use in binary form, without modification, are permitted provided
 * that the following conditions are met:
 *
 * *       No reverse engineering, decompilation, or disassembly of this software is
 * permitted with respect to any software provided in binary form.
 *
 * *       any redistribution and use are licensed by TI for use only with TI Devices.
 *
 * *       Nothing shall obligate TI to provide you with source code for the software
 * licensed and provided to you in object code.
 *
 * If software source code is provided to you, modification and redistribution of the
 * source code are permitted provided that the following conditions are met:
 *
 * *       any redistribution and use of the source code, including any resulting derivative
 * works, are licensed by TI for use only with TI Devices.
 *
 * *       any redistribution and use of any object code compiled from the source code
 * and any resulting derivative works, are licensed by TI for use only with TI Devices.
 *
 * Neither the name of Texas Instruments Incorporated nor the names of its suppliers
 *
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * DISCLAIMER.
 *
 * THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>
#include <stdint.h>
#include <TI/tivx.h>
#include <utils/app_init/include/app_init.h>
#include <utils/mem/include/app_mem.h>
#include <sys/time.h>

FILE *fp_src_img = NULL;
char file_name[20] = "file_name";
FILE *fp_dst_img[4] = {0};
uint32_t payload = 0;

void *virt_addr1 = NULL;
void *virt_addr2 = NULL;
uint64_t phys_addr1 = 0;
uint64_t phys_addr2 = 0;
uint8_t *src_buf = NULL, *dest_buf = NULL;
uint32_t width = 1280U;
uint32_t pitch = 4U*2*1280U;
uint32_t height = 1285U;
uint32_t size = 0;

uint8_t *image_test = NULL;

void timer_print_start(int task_id)
{
    struct timeval tv1;
    gettimeofday(&tv1, NULL);
    printf("DSS_M2M: task id %d start_time %ld s %ld us\n", task_id, tv1.tv_sec, tv1.tv_usec);
}

void timer_print_stop(int task_id)
{
    struct timeval tv1;
    gettimeofday(&tv1, NULL);
    printf("DSS_M2M: task id %d stop_time %ld s %ld us\n", task_id, tv1.tv_sec, tv1.tv_usec);
}

int main(int argc, char *argv[])
{
    int status = 0;
    width = 1280U;
    pitch = 4U*2*1280U;
    height = 1285U;
    size = pitch*height;

    status = appInit();

    if(status==0)
    {
        //int app_multi_cam_main(int argc, char* argv[]);

        //status = app_multi_cam_main(argc, argv);
        {
            extern void main_dss_m2m_test();

            virt_addr1 = appMemAlloc(APP_MEM_HEAP_DDR, size, 0);
            virt_addr2 = appMemAlloc(APP_MEM_HEAP_DDR, size, 0);

            if (virt_addr1 == NULL || virt_addr2 == NULL)
            {
                printf("UDMA: MEM alloc fail !");
            }

            src_buf = (uint8_t*)virt_addr1;
            dest_buf = (uint8_t*)virt_addr2;
            fp_src_img = fopen("img_raw.yuv", "r");
            fread(src_buf, width*pitch, 1, fp_src_img);
            //system("sync");
            sleep(2);
            main_dss_m2m_test();
            #if 0
            timer_print_start(7);
            for(int j = 0; j < 4; j++)
            {
                for(int i = 0; i < height; i++)
                {
                    memcpy(&dest_buf[j*width*2U*height + i*width*2U], &src_buf[j*width*2U + i*pitch], width*2U);
                }
            }
            timer_print_stop(7);
            #endif
            //system("sync");
            sleep(2);

            for(int i = 0; i < 4; i++)
            {
                sprintf(file_name, "img_raw_ch%d.yuv", i);
                fp_dst_img[i] = fopen(file_name, "w");
                fwrite(&dest_buf[i*width*height*2U], width*height*2U, 1, fp_dst_img[i]);
            }

        }



        appDeInit();
    }

    return status;
}
